#if 0 //To compile, type "nmake checkers.c"
checkers.exe: checkers.obj cwinmain.obj
	link       checkers.obj cwinmain.obj gdi32.lib user32.lib /opt:nowin98
	del checkers.obj
checkers.obj: checkers.c cwinmain.h; cl /c /TP checkers.c /Ox /G6F /MD /nologo
cwinmain.obj: cwinmain.c;            cl /c /TP cwinmain.c /Ox /G6F /MD /nologo
!if 0
#endif

/*
06/26/2005 06:32p 31,591 checkers.c <- old code using winmain.cpp/sysmain.h
05/13/2015 10:22p 23,220 checkers.c <- updated to use cwinmain.c/cwinmain.h

CHECKERS.C written by Ken Silverman (http://advsys.net/ken)
This code is designed to compile with Microsoft Visual C/C++ 6.0 or above.

 If you plan to use this code for your own project, all I ask is:
 1. You distribute your project free of charge - through the internet only. You are
		 not allowed to profit off this code in any way unless you consult me first.
 2. You give me proper credit. This is sufficient:
		 CHECKERS.C written by Ken Silverman (http://advsys.net/ken)

If you do anything cool with this code, don't hesitate to write me. I'd rather hear
	about it from you personally than find out about it on google. :)
*/


#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include "cwinmain.h"
#define PI 3.14159265358979323

	// Board layout (##'s are valid squares, 's are not)
	// 0 1 2 3
	// 4 5 6 7
	// 8 91011
	//12131415
	//16171819
	//20212223
	//24252627
	//28293031

typedef struct { int c[2], k, stat; } boardtyp;

#define ANIMSPEED 131072.0
#define COMPMOVES 12   //negative #'s means try to lose
#define KINGSC 3
#define PAWNSC 2
#define WINSC 256
static int domovesc[3];

enum { STARTMOVE,PLAYERMOVE,ANIMATE,GAMEOVER };
static int globmode = STARTMOVE, animval = 0, offx, offy, cellsiz = 64;
static int swapcols = 0, swapai = 0;

	//Timer global variables:
static double otim, tim = 0.0, dtim;

static boardtyp curboard;
static int neigh[32][4], neighb32[32][4], pow2[33], pow2b[33], npow2[32];

static _inline void dtol (double d, int *a)
{
	_asm
	{
		mov eax, a
		fld qword ptr d
		fistp dword ptr [eax]
	}
}

static const int pal16[] =
{
	0x000000,0x0000aa,0x00aa00,0x00aaaa,0xa00000,0xaa00aa,0x666644,0x909090,
	0x555555,0x5555ff,0x55ff55,0x55ffff,0xcc0000,0xff55ff,0xcccc99,0xdddddd
};

static _inline int bitrev (int b, int c)
{
	_asm
	{
		mov edx, b
		mov ecx, c
		xor eax, eax
 beg: shr edx, 1
		adc eax, eax
		dec ecx
		jnz short beg
	}
}

static _inline int bsr (int a) { _asm bsr eax, a }

	//similar to dotline2 from KCIRC.C
static void drawdotline2 (tiletype *dd, double x1, double y1, double x2, double y2, int col, int mskval)
{
	int i, j, inc, iend;
	double dx, dy;

	dx = x2-x1; dy = y2-y1;
	if (fabs(dx) >= fabs(dy))
	{
		if (dx == 0) return;
		if (x2 > x1) { dtol(x1,&i); dtol(x2,&iend); }
				  else { dtol(x2,&i); dtol(x1,&iend); mskval = _lrotl(bitrev(mskval,32),31); }
		dtol(1048576.0*dy/dx,&inc); dtol(y1*1048576.0+((double)i+.5-x1)*inc,&j);
		for(;i<iend;i++,j+=inc) { if (mskval&1) drawpix(dd,i,j>>20,col); mskval = _lrotl(mskval,1); }
	}
	else
	{
		if (y2 > y1) { dtol(y1,&i); dtol(y2,&iend); }
				  else { dtol(y2,&i); dtol(y1,&iend); mskval = _lrotl(bitrev(mskval,32),31); }
		dtol(1048576.0*dx/dy,&inc); dtol(x1*1048576.0+((double)i+.5-y1)*inc,&j);
		for(;i<iend;i++,j+=inc) { if (mskval&1) drawpix(dd,j>>20,i,col); mskval = _lrotl(mskval,1); }
	}
}

static void drawcircperim (tiletype *dd, int cx, int cy, int r, int col)
{
	int x, y, d;

	x = r; d = 0;
	for(y=0;y<=x;y++)
	{
		while (d >= 0) { d -= x; x--; d -= x; }
		d += y+y+1;
		drawpix(dd,cx-x,cy-y,col);
		drawpix(dd,cx-x,cy+y,col);
		drawpix(dd,cx+x,cy-y,col);
		drawpix(dd,cx+x,cy+y,col);
		drawpix(dd,cx-y,cy-x,col);
		drawpix(dd,cx-y,cy+x,col);
		drawpix(dd,cx+y,cy-x,col);
		drawpix(dd,cx+y,cy+x,col);
	}
}

static void drawcheck (tiletype *dd, int dax, int day, int col, int roy)
{
	if (!roy) //pawn
	{
		drawcircfill(dd, dax+1,day+2,(cellsiz>>1)-5,pal16[col]);
		drawcircfill(dd, dax-1,day-2,(cellsiz>>1)-5,pal16[col^8]);
		drawcircperim(dd,dax-1,day-2,(cellsiz>>1)-5,pal16[col]);
	}
	else //king
	{
		drawcircfill(dd, dax+1,day+2,(cellsiz>>1)-5,pal16[col]);
		drawcircfill(dd, dax-3,day-4,(cellsiz>>1)-5,pal16[col^8]);
		drawcircperim(dd,dax-1,day-1,(cellsiz>>1)-5,pal16[col^8]);
		drawcircperim(dd,dax-3,day-4,(cellsiz>>1)-5,pal16[col]);
		drawcircfill(dd, dax-3,day-4,((cellsiz*5)>>4)-5,pal16[col]);
		drawcircfill(dd, dax-2,day-1,((cellsiz*5)>>4)-5,pal16[col^8]);
		drawcircperim(dd,dax-3,day-4,((cellsiz*5)>>4)-5,pal16[col]);
	}
}

static void drawboard (tiletype *dd, boardtyp *b)
{
	int w, x, y, z, sy, daw, dax, day, col;

	for(z=0;z<32;z++)
	{
		y = (z>>2); w = ((z&3)<<1) + (y&1); x = (w^1);
		dax = x*cellsiz + offx; daw = w*cellsiz + offx;
		day = y*cellsiz + offy;
		for(sy=day;sy<day+cellsiz;sy++)
		{
			drawhlin(dd,dax,dax+cellsiz,sy,pal16[6]);
			drawhlin(dd,daw,daw+cellsiz,sy,pal16[14]);
		}
		col = -1;
		if (b->c[0]&pow2[z]) col = (swapcols^7);
		if (b->c[1]&pow2[z]) col = (swapcols^4);
		if (col >= 0) drawcheck(dd,dax+(cellsiz>>1),day+(cellsiz>>1),col,b->k&pow2[z]);
	}

		//Clear rest of screen to black
	for(y=0;y<offy;y++) drawhlin(dd,0,dd->x,y,pal16[0]);
	for(;y<offy+cellsiz*8;y++)
	{
		drawhlin(dd,0,offx,y,pal16[0]);
		drawhlin(dd,offx+cellsiz*8,dd->x,y,pal16[0]);
	}
	for(;y<dd->y;y++) drawhlin(dd,0,dd->x,y,pal16[0]);
}

static int domove (boardtyp *b, int m)
{
	int x, z0, z1, z2, z3, z4, sc, v, d, d0, d1;

	z0 = (m&65535); d = (m>>16);
	b->stat &= 1;
	z1 = neigh[z0][d];
	if (!((b->c[0]|b->c[1])&pow2[z1]))
	{
		x = (pow2[z0]|pow2[z1]);
		if (b->k&pow2[z0]) b->k ^= x;
		if (b->stat) b->c[1] ^= x; else b->c[0] ^= x;
		if ((~b->k)&pow2[z1]&0xf000000f) { b->k |= pow2[z1]; sc = domovesc[2]; } else sc = 0;
	}
	else
	{
		if (b->k&pow2[z1]) sc = domovesc[1]; else sc = domovesc[0];
		z2 = neigh[z1][d]; x = (pow2[z0]|pow2[z2]);
		if (b->k&pow2[z0]) b->k ^= x;
		b->k &= npow2[z1];
		if (b->stat) { b->c[0] ^= pow2[z1]; b->c[1] ^= x; }
				  else { b->c[1] ^= pow2[z1]; b->c[0] ^= x; }
		if ((~b->k)&pow2[z2]&0xf000000f)
			{ b->k |= pow2[z2]; sc += domovesc[2]; }
		else
		{
				//Check for multiple jumps (if so, other player loses turn)
			if (pow2[z2]&b->k) { d0 = 0; d1 = 4; } //is it a king?
			else { d0 = (b->stat<<1); d1 = d0+2; }
			for(d=d0;d<d1;d++)
			{
				z3 = neigh[z2][d]; if (z3 < 0) continue;
				if (b->stat) v = b->c[0]; else v = b->c[1];
				if (pow2[z3]&v)
				{
					z4 = neigh[z3][d]; if (z4 < 0) continue;
					if (pow2[z4]&(b->c[0]|b->c[1])) continue;
					b->stat += (z2<<16)+2;
					if (b->stat&1) return(-sc); else return(sc); //Double jump possible!
				}
			}
		}
	}
	b->stat ^= 1;
	if (b->stat) return(sc); else return(-sc);
}

static int domove_nomodify (boardtyp *b, int m)
{
	int z0, z1, d, sc;

	z0 = (m&65535); d = (m>>16);
	z1 = neigh[z0][d];
	if ((b->c[0]|b->c[1])&pow2[z1])
	{
		if (b->k&pow2[z1]) sc = domovesc[1]; else sc = domovesc[0];
		z1 = neigh[z1][d];
	} else sc = 0;
	if ((!(b->k&pow2[z0])) && (pow2[z1]&0xf000000f)) sc += domovesc[2];
	if (b->stat&1) return(-sc); else return(sc);
}


static int getvalmovs (boardtyp *b, int *valbuf)
{
	int c, d, d0, d1, z0, z1, z2, z3, j, u, v, w;

	if (b->stat&1) { u = b->c[1]; v = b->c[0]; } else { u = b->c[0]; v = b->c[1]; }
	w = (b->c[0]|b->c[1]);

	c = 0;

		//Later part of double(or triple, etc...) jump
	if (b->stat&2)
	{
		z1 = (b->stat>>16);
		if (pow2[z1]&b->k) { d0 = 0; d1 = 4; } //is it a king?
		else { d0 = ((b->stat&1)<<1); d1 = d0+2; }
		for(d=d0;d<d1;d++)
		{
#if 0
			z2 = neigh[z1][d]; if (z2 < 0) continue;
			if (!(pow2[z2]&v)) continue;
			z3 = neigh[z2][d]; if (z3 < 0) continue;
			if (pow2[z3]&w) continue;
			valbuf[c++] = (d<<16)+z1;
#else
			z2 = neighb32[z1][d]; if ((!(pow2[z2]&v)) || (pow2b[neighb32[z2][d]]&w)) continue;
			valbuf[c++] = (d<<16)+z1;
#endif
		}
		return(c);
	}

	j = 0;
	while (u)
	{
		z0 = bsr(u); u &= npow2[z0];

		if (pow2[z0]&b->k) { d0 = 0; d1 = 4; } //is it a king?
		else { d0 = (b->stat<<1); d1 = d0+2; }
		for(d=d0;d<d1;d++)
		{
			z1 = neigh[z0][d]; if (z1 < 0) continue;
			if (!(pow2[z1]&w))
			{
				if (!j) valbuf[c++] = (d<<16) + z0;
				continue;
			}
			if (!(pow2[z1]&v)) continue;

			//z2 = neigh[z1][d]; if (z2 < 0) continue;
			//if (pow2[z2]&w) continue;
			if (pow2b[neighb32[z1][d]]&w) continue;

			if (!j) { c = 0; j = 1; }
			valbuf[c++] = (d<<16) + z0;
		}
	}
	return(c);
}

#define MAXREC 64
static int endsc[MAXREC+1], nendsc[MAXREC+1];

#define USEHASH 1
#if (USEHASH != 0)
#define CACSIZ 1048576 //1048576*24
#define HASHEAD 8192   //8192
typedef struct { int n, sc; boardtyp b; } hashtyp;
static int hashead[MAXREC][HASHEAD], cacnum, ghashv; //0.3115 sec
static hashtyp hashtab[CACSIZ];
static int hashlastlev = 0;

static void inithash (int numoves)
{
	memset(hashead,-1,sizeof(hashead[0][0])*HASHEAD*numoves);
	cacnum = 0;
}

static int gethash (boardtyp *b, int dir)
{
	int i;
	unsigned int v;
	v = _lrotl(b->c[0],7)-_lrotl(b->c[1],12)+b->k-b->stat;
	v = (((v>>20)-(v>>11)-v)%((unsigned int)HASHEAD));
	for(i=hashead[dir-1][v];i>=0;i=hashtab[i].n)
		if ((hashtab[i].b.c[0] == b->c[0]) && (hashtab[i].b.c[1] == b->c[1]) &&
			 (hashtab[i].b.k == b->k) && (hashtab[i].b.stat == b->stat))
				return(hashtab[i].sc);
	ghashv = v;
	return(0x80000000);
}

static void puthash (boardtyp *b, int dir, int sc)
{
	if (cacnum >= CACSIZ) return;
	hashtab[cacnum].n = hashead[dir-1][ghashv];
							  hashead[dir-1][ghashv] = cacnum;
	hashtab[cacnum].sc = sc;
	hashtab[cacnum].b.c[0] = b->c[0];
	hashtab[cacnum].b.c[1] = b->c[1];
	hashtab[cacnum].b.k    = b->k;
	hashtab[cacnum].b.stat = b->stat;
	cacnum++;
}
#endif

static int recursemin (boardtyp *b, int *vm, int vmc, int dir, int osc, int obestsc);
static int recursemax (boardtyp *b, int *vm, int vmc, int dir, int osc, int obestsc)
{
	int z, sc, nsc, bestsc, tvmc, tvm[48];
	boardtyp nb;

	if (dir <= 0)
	{
		bestsc = nendsc[dir];
		for(z=vmc-1;z>=0;z--)
		{
			sc = domove_nomodify(b,vm[z])+osc;
			if (sc > bestsc) { bestsc = sc; if (sc >= obestsc) break; }
		}
		return(bestsc);
	}
#if (USEHASH != 0)
	if (dir >= hashlastlev) { sc = gethash(b,dir); if (sc != 0x80000000) return(sc); }
#endif
	bestsc = nendsc[dir];
	for(z=vmc-1;z>=0;z--)
	{
		nb.c[0] = b->c[0]; nb.c[1] = b->c[1]; nb.k = b->k; nb.stat = b->stat; nsc = domove(&nb,vm[z])+osc;
		tvmc = getvalmovs(&nb,tvm);
		if (nb.stat&2) sc = recursemax(&nb,tvm,tvmc,dir,nsc,bestsc);
					 else sc = recursemin(&nb,tvm,tvmc,dir-1,nsc,bestsc);
		if (sc > bestsc) { bestsc = sc; if (sc >= obestsc) break; /*alpha-beta pruning*/ }
	}
#if (USEHASH != 0)
	if (dir >= hashlastlev) puthash(b,dir,bestsc);
#endif
	return(bestsc);
}

static int recursemin (boardtyp *b, int *vm, int vmc, int dir, int osc, int obestsc)
{
	int z, sc, nsc, bestsc, tvmc, tvm[48];
	boardtyp nb;

	if (dir <= 0)
	{
		bestsc = endsc[dir];
		for(z=vmc-1;z>=0;z--)
		{
			sc = domove_nomodify(b,vm[z])+osc;
			if (sc < bestsc) { bestsc = sc; if (sc <= obestsc) break; }
		}
		return(bestsc);
	}
#if (USEHASH != 0)
	if (dir >= hashlastlev) { sc = gethash(b,dir); if (sc != 0x80000000) return(sc); }
#endif
	bestsc = endsc[dir];
	for(z=vmc-1;z>=0;z--)
	{
		nb.c[0] = b->c[0]; nb.c[1] = b->c[1]; nb.k = b->k; nb.stat = b->stat; nsc = domove(&nb,vm[z])+osc;
		tvmc = getvalmovs(&nb,tvm);
		if (nb.stat&2) sc = recursemin(&nb,tvm,tvmc,dir,nsc,bestsc);
					 else sc = recursemax(&nb,tvm,tvmc,dir-1,nsc,bestsc);
		if (sc < bestsc) { bestsc = sc; if (sc <= obestsc) break; /*alpha-beta pruning*/ }
	}
#if (USEHASH != 0)
	if (dir >= hashlastlev) puthash(b,dir,bestsc);
#endif
	return(bestsc);
}

	//numoves: >0: # moves looking ahead (+1), trying to win
	//          0: completely random moving (no intelligence)
	//         <0: -# moves looking ahead (+1), trying to lose
static int getcompmove (boardtyp *b, int *valbuf, int vmc, int numoves)
{
	int i, z, sc, nsc, tvmc, vm[48], tvm[48], bestsc, bestz, try2lose;
	boardtyp nb;

	if (!numoves) return(valbuf[(rand()*vmc)>>15]); //Completely random move
	if (numoves < 0) { try2lose = 1; numoves = -numoves; } else try2lose = 0;
	if (vmc == 1) return(valbuf[0]);

	if (numoves > MAXREC) numoves = MAXREC;
	for(endsc[0]=WINSC,z=0;z<numoves;z++) endsc[z+1] = (endsc[z]<<1);
	for(z=0;z<=numoves;z++) nendsc[z] = -endsc[z];
#if (USEHASH != 0)
	inithash(numoves);
#endif

	hashlastlev = numoves-10;

		//Shuffle valid moves array (choose random move of equally best list)
	for(z=vmc-1;z>1;z--)
	{
		i = ((rand()*z)>>15);
		sc = valbuf[z]; valbuf[z] = valbuf[i]; valbuf[i] = sc;
	}

	if (!(curboard.stat&1)) { domovesc[0] = PAWNSC; domovesc[1] = KINGSC; domovesc[2] = KINGSC-PAWNSC; }
							 else { domovesc[0] =-PAWNSC; domovesc[1] =-KINGSC; domovesc[2] = PAWNSC-KINGSC; }

	bestz = 0;
	if (!try2lose)
	{
		bestsc = nendsc[numoves];
		for(z=vmc-1;z>=0;z--)
		{
			nb.c[0] = b->c[0]; nb.c[1] = b->c[1]; nb.k = b->k; nb.stat = b->stat; nsc = domove(&nb,valbuf[z]);
			tvmc = getvalmovs(&nb,tvm);
			if (nb.stat&2) sc = recursemax(&nb,tvm,tvmc,numoves,nsc,bestsc);
						 else sc = recursemin(&nb,tvm,tvmc,numoves-1,nsc,bestsc);
			if (sc > bestsc) { bestsc = sc; bestz = z; }
		}
	}
	else
	{
		bestsc = endsc[numoves];
		for(z=vmc-1;z>=0;z--)
		{
			nb.c[0] = b->c[0]; nb.c[1] = b->c[1]; nb.k = b->k; nb.stat = b->stat; nsc = domove(&nb,valbuf[z]);
			tvmc = getvalmovs(&nb,tvm);
			if (nb.stat&2) sc = recursemin(&nb,tvm,tvmc,numoves,nsc,bestsc);
						 else sc = recursemax(&nb,tvm,tvmc,numoves-1,nsc,bestsc);
			if (sc < bestsc) { bestsc = sc; bestz = z; }
		}
	}
	return(valbuf[bestz]);
}

int WINAPI WinMain (HINSTANCE hinst, HINSTANCE hpinst, LPSTR cmdline, int ncmdshow)
{
	boardtyp tboard, bb, bbb;
	tiletype dd;
	double comptim = 0;
	int omousx, omousy, selm, valmovcnt, valmov[48]; //5 bits (start pos) + 2 bits (dir)
	int i, j, k, x, y, z, xx, yy, zz, tvalmov[48], msc[48], obstatus = 0;
	int dx, dy, dax, day, x0, y0, z0, x1, y1, z1, col, t, d, m, jmp, seld, argc;
	char ch, *argv[MAX_PATH>>1];

	xres = 304; yres = 304; prognam = "Checkers by Ken Silverman";

	argc = cmdline2arg(cmdline,argv);
	for(i=1;i<argc;i++)
	{
		if (argv[i][0] != '/') continue;
		if ((argv[i][1] >= '0') && (argv[i][1] <= '9'))
		{
			k = 0; ch = 0;
			for(j=1;;j++)
			{
				if ((argv[i][j] >= '0') && (argv[i][j] <= '9'))
					{ k = (k*10+argv[i][j]-48); continue; }
				switch (ch)
				{
					case 0: xres = k; break;
					case 1: yres = k; break;
				}
				if (!argv[i][j]) break;
				ch++; if (ch > 2) break;
				k = 0;
			}
		}
	}

	if (!initapp(hinst)) return(-1);

	for(z=0;z<32;z++) { pow2[z] = pow2b[z] = (1<<z); npow2[z] = ~pow2[z]; } pow2[32] = 0; pow2b[32] = -1;
	for(z=0;z<32;z++)
	{
		y = (z>>2); x = ((z&3)<<1)+1-(y&1);
		for(zz=0;zz<4;zz++) neigh[z][zz] = -1;
		for(zz=z-5;zz<=z+5;zz++)
		{
			if ((unsigned int)zz >= 32) continue;
			yy = (zz>>2); xx = ((zz&3)<<1)+1-(yy&1);
			if ((labs(xx-x) == 1) && (labs(yy-y) == 1))
				neigh[z][((yy>y)<<1)+(xx>x)] = zz;
		}
		for(zz=0;zz<4;zz++)
		{
			if (neigh[z][zz] < 0) neighb32[z][zz] = 32;
								  else neighb32[z][zz] = neigh[z][zz];
		}
	}

	srand(GetTickCount());

	curboard.c[0] = 0xfff00000;   //1 = white
	curboard.c[1] = 0x00000fff;   //1 = red
	curboard.k    = 0x00000000;   //1 = king
	curboard.stat = 0;            //bit 0: turn, bit 1: "must move king state"
											//bit 16-20: which piece must move (if bit 1 == 1)

	tim = klock();

	while (!breath())
	{
		Sleep(1);

		if (keystatus[1]) { quitloop(); }
		otim = tim; tim = klock(); dtim = tim-otim;

		if (keystatus[0x29]) { keystatus[0x29] = 0; swapcols ^= (4^7); } //   ` Swap White & Red checkers
		if (keystatus[0x0f]) //Tab: Swap player & computer AI
		{
			keystatus[0x0f] = 0; swapai ^= 1;
			if (((curboard.stat&1) != swapai) && (globmode == PLAYERMOVE)) //Let computer take over immediately...
			{
				globmode = ANIMATE; selm = getcompmove(&curboard,valmov,valmovcnt,COMPMOVES);
				animval = 0; otim = klock(); comptim = otim-tim; tim = otim;
			}
		}
		if (keystatus[0xd3]) //Del: pass turn to other side (only possible before 1st move)
		{
			keystatus[0xd3] = 0;
			if ((curboard.c[0] == 0xfff00000) &&
				 (curboard.c[1] == 0x00000fff) &&
				 (curboard.k    == 0x00000000) && (globmode == PLAYERMOVE))
				{ curboard.stat ^= 1; swapai ^= 1; globmode = STARTMOVE; }
		}

		switch(globmode)
		{
			case STARTMOVE:
				valmovcnt = getvalmovs(&curboard,valmov);
				if (!valmovcnt) globmode = GAMEOVER;
				//else if (valmovcnt == 1) { selm = valmov[0]; globmode = ANIMATE; animval = 0; } //Only 1 valid move!
				else
				{
					if ((curboard.stat&1) == swapai)
					{
						globmode = PLAYERMOVE; selm = -1;
					}
					else
					{
						globmode = ANIMATE; selm = getcompmove(&curboard,valmov,valmovcnt,COMPMOVES);
						animval = 0; otim = klock(); comptim = otim-tim; tim = otim;
					}
				}
				break;
			case PLAYERMOVE:
				if (keystatus[0xcb]|keystatus[0xcd]|keystatus[0xc8]|keystatus[0xd0])
				{
					POINT p0;
					if (keystatus[0xcb]) { keystatus[0xcb] = 0; mousx = ((mousx-cellsiz-offx-(cellsiz>>1))/cellsiz)*cellsiz + offx + (cellsiz>>1); }
					if (keystatus[0xcd]) { keystatus[0xcd] = 0; mousx = ((mousx+cellsiz-offx-(cellsiz>>1))/cellsiz)*cellsiz + offx + (cellsiz>>1); }
					if (keystatus[0xc8]) { keystatus[0xc8] = 0; mousy = ((mousy-cellsiz-offy-(cellsiz>>1))/cellsiz)*cellsiz + offy + (cellsiz>>1); }
					if (keystatus[0xd0]) { keystatus[0xd0] = 0; mousy = ((mousy+cellsiz-offy-(cellsiz>>1))/cellsiz)*cellsiz + offy + (cellsiz>>1); }
					p0.x = 0; p0.y = 0; ClientToScreen(ghwnd,&p0);
					SetCursorPos(p0.x+mousx,p0.y+mousy);
				}

				if (((bstatus&1) > (obstatus&1)) || (keystatus[0x39]))
				{
					keystatus[0x39] = 0;
					if (selm < 0)
					{
						if ((((mousx-offx)/cellsiz)^((mousy-offy)/cellsiz))&1)
						{
							selm = (((mousy-offy)/cellsiz)<<2) + ((mousx-offx)/(cellsiz*2));
							for(z=valmovcnt-1;z>=0;z--) if ((valmov[z]&65535) == selm) break;
							if (z < 0) selm = -1;
							omousx = mousx; omousy = mousy;
						}
					}
				}
				if (((bstatus&1) < (obstatus&1)) || (keystatus[0x1c]|keystatus[0x9c]))
				{
					keystatus[0x1c] = 0; keystatus[0x9c] = 0;
					if (((mousx-offx)/cellsiz) != ((omousx-offx)/cellsiz) && ((mousy-offy)/cellsiz) != ((omousy-offy)/cellsiz))
					{
						if (((mousx-offx)/cellsiz) < ((omousx-offx)/cellsiz)) seld = 0; else seld = 1;
						if (((mousy-offy)/cellsiz) > ((omousy-offy)/cellsiz)) seld += 2;
						m = (seld<<16) + selm;
						for(z=valmovcnt-1;z>=0;z--) if (valmov[z] == m) break;
						if (z >= 0) { domove(&curboard,(seld<<16)+selm); globmode = STARTMOVE; }
					}
					selm = -1;
				}
				break;
			case GAMEOVER:
				if (keystatus[0x15]) //Y
				{
					keystatus[0x15] = 0;
					curboard.c[0] = 0xfff00000; curboard.c[1] = 0x00000fff;
					curboard.k    = 0x00000000; curboard.stat = (rand()&1);
					globmode = STARTMOVE;
				}
				if (keystatus[0x31]) { keystatus[0x31] = 0; quitloop(); } //N
				break;
		}

		if (startdirectdraw(&dd.f,&dd.p,&dd.x,&dd.y))
		{
			cellsiz = (min(dd.x,dd.y)>>3);
			offx = ((dd.x - (cellsiz<<3))>>1);
			offy = ((dd.y - (cellsiz<<3))>>1);

			switch(globmode)
			{
				case STARTMOVE: drawboard(&dd,&curboard); break;
				case PLAYERMOVE:
					bb.c[0] = curboard.c[0]; bb.c[1] = curboard.c[1]; bb.k = curboard.k; bb.stat = curboard.stat;
					if (selm >= 0)
					{
						bb.c[0] &= npow2[selm];
						bb.c[1] &= npow2[selm];
					}
					drawboard(&dd,&bb);
					if (selm >= 0)
					{
						if (curboard.c[0]&pow2[selm]) col = 7; else col = 4;
						drawcheck(&dd,mousx,mousy,col,curboard.k&pow2[selm]);
					}

					for(z=0;z<valmovcnt;z++)
					{
						m = valmov[z]; z0 = (m&65535); d = (m>>16);
	
						y = (z0>>2); x = ((z0&3)<<1)+1-(y&1);
						dx = ((d&1)<<1)-1; dy = (d&2)-1;
	
						z1 = neigh[z0][d];
						x0 = x*cellsiz + (cellsiz>>1) + offx + ((dx*cellsiz)>>2);
						y0 = y*cellsiz + (cellsiz>>1) + offy + ((dy*cellsiz)>>2);
						if (!((curboard.c[0]|curboard.c[1])&pow2[z1]))
						{
							x1 = x*cellsiz + ((dx*cellsiz*3)>>2) + (cellsiz>>1) + offx;
							y1 = y*cellsiz + ((dy*cellsiz*3)>>2) + (cellsiz>>1) + offy;
						}
						else
						{
							x1 = x*cellsiz + ((dx*cellsiz*7)>>2) + (cellsiz>>1) + offx;
							y1 = y*cellsiz + ((dy*cellsiz*7)>>2) + (cellsiz>>1) + offy;
						}
						drawdotline2(&dd,x0  ,y0,x1  ,y1,pal16[11],_lrotr(0x3c3c3c3c,(int)(tim*16.0)));
						drawdotline2(&dd,x0-1,y0,x1-1,y1,pal16[11],_lrotr(0x3c3c3c3c,(int)(tim*16.0)));
						drawdotline2(&dd,x0+1,y0,x1+1,y1,pal16[11],_lrotr(0x3c3c3c3c,(int)(tim*16.0)));
						drawdotline2(&dd,x0,y0-1,x1,y1-1,pal16[11],_lrotr(0x3c3c3c3c,(int)(tim*16.0)));
						drawdotline2(&dd,x0,y0+1,x1,y1+1,pal16[11],_lrotr(0x3c3c3c3c,(int)(tim*16.0)));
					}

					//drawdotline2(&dd,mousx-6,mousy-5,mousx+4,mousy+5,pal16[15],-1);
					//drawdotline2(&dd,mousx+4,mousy-5,mousx-6,mousy+5,pal16[15],-1);
					//drawdotline2(&dd,mousx-5,mousy-5,mousx+5,mousy+5,pal16[0],-1);
					//drawdotline2(&dd,mousx+5,mousy-5,mousx-5,mousy+5,pal16[0],-1);
					break;
				case ANIMATE:
					dtol((tim-otim)*ANIMSPEED,&t); animval = min(animval+t,65536);

					z0 = (selm&65535); d = (selm>>16);
					jmp = ((pow2[neigh[z0][d]]&(curboard.c[0]|curboard.c[1])) != 0);

					y = (z0>>2); x = ((z0&3)<<1) + 1 - (y&1);
					dx = ((d&1)<<1)-1; dy = (d&2)-1;

					tboard.c[0] = (npow2[z0]&curboard.c[0]);
					tboard.c[1] = (npow2[z0]&curboard.c[1]);
					tboard.k = curboard.k;

						//Erase jumped checker in middle of animation
					if ((jmp) && (animval >= 32768))
					{
						z1 = neigh[z0][d];
						tboard.c[0] &= npow2[z1];
						tboard.c[1] &= npow2[z1];
					}

					drawboard(&dd,&tboard);

						//Draw interpolated/animated checker
					dax = x*cellsiz + (1.0-cos((double)animval*PI/65536))*.5*(double)(dx*(jmp*cellsiz + cellsiz)) + (cellsiz>>1) + offx;
					day = y*cellsiz + (1.0-cos((double)animval*PI/65536))*.5*(double)(dy*(jmp*cellsiz + cellsiz)) + (cellsiz>>1) + offy;

					if (pow2[z0]&curboard.c[0]) col = 7; else col = 4;
					drawcheck(&dd,dax,day,col,pow2[z0]&curboard.k);

					if (animval >= 65536) { domove(&curboard,selm); globmode = STARTMOVE; }
					break;
				case GAMEOVER:
					drawboard(&dd,&curboard);
					if (!(curboard.stat&1)) print6x8(&dd,(dd.x>>1)-9*3,(dd.y>>1)-4-8,pal16[0],pal16[12],"Red wins!");
											 else print6x8(&dd,(dd.x>>1)-11*3,(dd.y>>1)-4-8,pal16[0],pal16[15],"White wins!");
					print6x8(&dd,(dd.x>>1)-17*3,(dd.y>>1)-4+8,pal16[0],pal16[15],"Play again? (Y/N)");
					break;
			}

			print6x8(&dd,0,0,pal16[0],-1,"%.4f",comptim);

			stopdirectdraw();
			nextpage();
		}
		obstatus = bstatus;
	}

	uninitapp();
	return(0);
}

#if 0
!endif
#endif
